/* bastard api -- these should be called from the macro language and the prog */
#ifndef BASTARD_API_INCLUDE /* prevent redeclaration errors in scripts */
   #define BASTARD_API_INCLUDE
   #define INCLUDED_FROM_BASTARD_H

   #ifndef PATH_MAX
      #include <dirent.h>
   #endif

/* Contents:_________________________________________________________________
 1.  Low-Level DB Interface
 2.  Generic Object Access
 3.  Low-Level Disassembler Access
 4.  Loading
 5.  Saving
 6.  Sections
 7.  Disassembly
 8.  Addresses
 9.  Names & Comments
 10. Structures & Constants
 11. Code
 12. Strings
 13. Imports
 14. Exports
 15. XRefs
 16. Functions
 17. Macros
 18. Search
 19. System Interaction                                                  */

/* Notes:_________________________________________________________________
 * + The following naming conventions are used:
 *       rva       Relative Virtual Address -- the address of a location of
 *                   code or data once the program has been loaded by the OS
 *       pa        Physical Address -- the offset of a location of code or
 *                   data within the executable file/image.
 *
 * + Most routines return 0 on error or failure; the reason for the error can
 *   be found by calling GetLastError();
 *   
 * + Routines that take character arrays as arguments generally do not require
 *   the length of the array. This because the strings in the DB are of
 *   fixed length [see bdb.h], and thus it simple enough for the caller to
 *   know the size of array they should use. Naturally this goes against all
 *   sane security practice; however anyone incorporating a program such as
 *   the bastard into their firewalling policy is pretty much beyond saving.
 *   Be warned: you are expected to know what you are doing.
 */

/* DB Interface -- db.c
   ======================================================================== */
/* These are located in db.c, but are included here for the convenience of
 * the script writers who shouldn't really be using them, but probably will
 * regardless */

/* DB State -- a way to keep the DB from changing the current record inside
 *             a subroutine. Wrap all calls to the DB as follows:
 *                int *state;
 *                state = DBSaveState();
 *                 ...
 *                DBRestoreState(state);
 *             and you should be alright. */
   void* db_save_state();
   int db_restore_state(void *state);
   /* ---- Low-level DB access ---- */
   /*   These expect *dest to be pre-allocated */
   int db_index_first(int index, void *dest);
   int db_index_next(int index, void *dest);
   int db_index_prev(int index, void *dest);
   int db_index_last(int index, void *dest);
   int db_index_find(int index, void *value, void *dest);
   int db_table_first(int table, void *dest);
   int db_table_next(int table, void *dest);
   int db_table_prev(int table, void *dest);
   int db_table_last(int table, void *dest);
   int db_record_insert(int table, void *src);
   int db_record_update(int index, void *value, void *src);
   int db_record_delete(int index, void *value);
   int db_find_closest_prev(int index, void *value, void *dest);
   int db_find_closest_next(int index, void *value, void *dest);
   int db_table_count(int table);
   /* ---- generic 'object' access ---- */
   /*   These calloc a struct and fill it; it is up to the caller to free */
   struct address * GetAddressObject(long rva);
   struct code * GetCodeObject(long rva);
   struct addr_exp * GetAddrExpObject(int id);
   struct code_effect * GetCodeEffectObject(int id);
   struct section * GetSectionObject(char *name);
   struct xref * GetXrefObject(int id);
   struct int_code * GetIntCodeObject(int id);
   struct fin_code * GetFinCodeObject(int id);
   struct export_addr * GetExportObject(long rva);
   struct import_addr * GetImportObject(long rva);
   struct string * GetStringObject(long rva);
   struct name * GetNameObject(long rva);
   struct comment * GetCommentObject(int id);
   struct function * GetFunctionObject(int id);
   struct func_param * GetFuncParamObject(int id);
   struct func_local * GetFuncLocalObject(int id);
   struct func_effect * GetFuncEffectObject(int id);
   struct f_inline * GetFInlineObject(int id);
   struct structure * GetStructureObject(int id);
   struct struct_member * GetStructMemberObject(int id);
   struct data_type * GetDataTypeObject(int id);
   struct constant * GetConstantObject(int id);
   struct bc_macro * GetBCMacroObject(int id);

   typedef void (*BAPI_CALLBACK)(void *item, void *data);
   void db_index_foreach(int index, BAPI_CALLBACK func, void *data);
   /* Bastard Extensions -- extension.c
      ===================================================================== */
   #include <api/api_plugin.h>


   /* Loading and Saving -- api_loadsave.c
      ===================================================================== */
   #include <api/api_target.h>

   /* Sections -- api_section.c
      ===================================================================== */
   #include <api/api_section.h>

   /* Disassembly -- api_disasm.c
      ===================================================================== */
   #include <api/api_disasm.h>

   /* Addresses -- api_address.c
      ===================================================================== */
   #include <api/api_addr.h>

   /* Names & Comments -- api_namecmt.c
      ===================================================================== */
   #include <api/api_name.h>
   #include <api/api_comment.h>

   /* Structures & Constants -- api_structconst.c
      ===================================================================== */
   #include <api/api_dtype.h>
   #include <api/api_struct.h>
   #include <api/api_const.h>

   /* Code -- api_code.c
      ===================================================================== */
   #include <api/api_code.h>

   /*   Address Expressions -- api_code.c
      ===================================================================== */
   #include <api/api_addrexp.h>

   /*   Strings -- api_strings.c
      ===================================================================== */
   #include <api/api_string.h>

   /* Imports -- api_importexport.c
      ===================================================================== */
   #include <api/api_imp.h>
   #include <api/api_lib.h>

   /* Exports -- api_importexport.c
      ===================================================================== */
   #include <api/api_exp.h>

   /* XRefs -- api_xref.c
      ===================================================================== */
   #include <api/api_xref.h>

   /* Functions -- api_functions.c
      ===================================================================== */
   #include <api/api_func.h>

   /* Macros -- api_macros.c
      ===================================================================== */
   #include <api/api_macro.h>

   /* Search -- api_search.c
      ===================================================================== */
   #include <api/api_search.h>
			 
   /* Bastard Environment -- api_env.c
      ===================================================================== */
   #include <api/api_env.h>

   /* System Interaction -- api_system.c
      ===================================================================== */
   #include <api/api_sys.h>


#endif /* prevent redeclaration errors in scripts */
